@@ -33,19 +33,19 @@
 #include "gnome-vfs-mime.h"
 #include "gnome-vfs-private-utils.h"
 #include "gnome-vfs-result.h"
-#include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <string.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <time.h>
 #include <unistd.h>
 
+#include <glib/gstdio.h>
+
 #if !defined getc_unlocked && !defined HAVE_GETC_UNLOCKED
 # define getc_unlocked(fp) getc (fp)
 #endif
@@ -83,10 +83,8 @@ static ApplicationRegistryDir user_registry_dir;
 /* To initialize the module automatically */
 static gboolean gnome_vfs_application_registry_initialized = FALSE;
 
-
-static GList *current_lang = NULL;
-/* we want to replace the previous key if the current key has a higher
-   language level */
+/* we want to replace the previous key if the current key has a lower
+   (ie: better) language level */
 static char *previous_key = NULL;
 static int previous_key_lang_level = -1;
 
@@ -561,27 +559,28 @@ application_sync (Application *application, FILE *fp)
 	fprintf (fp, "\n");
 }
 
-
 /* this gives us a number of the language in the current language list,
-   the higher the number the "better" the translation */
+   the lower the number the "better" the translation */
 static int
 language_level (const char *lang)
 {
+	const char * const *lang_list;
 	int i;
-	GList *li;
 
 	if (lang == NULL)
-		return 0;
+		lang = "C";
 
-	for (i = 1, li = current_lang; li != NULL; i++, li = g_list_next (li)) {
-		if (strcmp ((const char *) li->data, lang) == 0)
+	/* The returned list is sorted from most desirable to least
+            desirable and always contains the default locale "C". */
+	lang_list = g_get_language_names();
+
+	for (i = 0; lang_list[i]; i++)
+		if (strcmp (lang_list[i], lang) == 0)
 			return i;
-	}
 
 	return -1;
 }
 
-
 static void
 application_add_key (Application *application, const char *key,
 		     const char *lang, const char *value)
@@ -639,7 +638,7 @@ application_add_key (Application *application, const char *key,
 	    /* our language level really sucks and the previous
 	       translation was of better language quality so just
 	       ignore us */
-	    previous_key_lang_level > lang_level) {
+	    previous_key_lang_level < lang_level) {
 		return;
 	}
 
@@ -706,7 +705,7 @@ load_application_info_from (const char *filename, gboolean user_owned)
 	char *key;
 	char *lang;
 
-	fp = fopen (filename, "r");
+	fp = g_fopen (filename, "r");
 	if (fp == NULL)
 		return;
 
@@ -867,53 +866,57 @@ load_application_info_from (const char *filename, gboolean user_owned)
 static void
 application_info_load (ApplicationRegistryDir *source)
 {
-	DIR *dir;
-	struct dirent *dent;
+	GDir *dir;
+	const char *dent;
 	const int extlen = sizeof (".applications") - 1;
 	char *filename;
 	struct stat s;
 
-	if (stat (source->dirname, &s) != -1)
+	if (g_stat (source->dirname, &s) != -1)
 		source->valid = TRUE;
 	else
 		source->valid = FALSE;
 	
-	dir = opendir (source->dirname);
+	dir = g_dir_open (source->dirname, 0, NULL);
 	if (dir == NULL) {
 		source->valid = FALSE;
 		return;
 	}
 	if (source->system_dir) {
-		filename = g_strconcat (source->dirname, "/gnome-vfs.applications", NULL);
+		filename = g_build_filename (source->dirname,
+					     "gnome-vfs.applications",
+					     NULL);
 		load_application_info_from (filename, FALSE /*user_owned*/);
 		g_free (filename);
 	}
 
-	while ((dent = readdir (dir)) != NULL){
+	while ((dent = g_dir_read_name (dir)) != NULL){
 		
-		int len = strlen (dent->d_name);
+		int len = strlen (dent);
 
 		if (len <= extlen)
 			continue;
-		if (strcmp (dent->d_name + len - extlen, ".applications"))
+		if (strcmp (dent + len - extlen, ".applications"))
 			continue;
-		if (source->system_dir && strcmp (dent->d_name, "gnome-vfs.applications") == 0)
+		if (source->system_dir && strcmp (dent, "gnome-vfs.applications") == 0)
 			continue;
-		if ( ! source->system_dir && strcmp (dent->d_name, "user.applications") == 0)
+		if ( ! source->system_dir && strcmp (dent, "user.applications") == 0)
 			continue;
-		filename = g_strconcat (source->dirname, "/", dent->d_name, NULL);
+		filename = g_build_filename (source->dirname, dent, NULL);
 		load_application_info_from (filename, FALSE /*user_owned*/);
 		g_free (filename);
 	}
 
 	if ( ! source->system_dir) {
-		filename = g_strconcat (source->dirname, "/user.applications", NULL);
-		/* Currently this is the only file that is "user owned".  It actually makes
-		 * sense.  Editting of other files from the API would be too complex */
+		filename = g_build_filename (source->dirname, "user.applications", NULL);
+		/* Currently this is the only file that is "user
+		 * owned".  It actually makes sense.  Editting of
+		 * other files from the API would be too complex
+		 */
 		load_application_info_from (filename, TRUE /*user_owned*/);
 		g_free (filename);
 	}
-	closedir (dir);
+	g_dir_close (dir);
 
 	_gnome_vfs_file_date_tracker_start_tracking_file (registry_date_tracker, source->dirname);
 }
@@ -952,20 +955,23 @@ gnome_vfs_application_registry_init (void)
 	generic_mime_types  = g_hash_table_new (g_str_hash, g_str_equal);
 	specific_mime_types  = g_hash_table_new (g_str_hash, g_str_equal);
 	
-	current_lang = gnome_vfs_i18n_get_language_list ("LC_MESSAGES");
-
 	/*
 	 * Setup the descriptors for the information loading
 	 */
 
-	gnome_registry_dir.dirname = g_strdup (DATADIR "/application-registry");
+	gnome_registry_dir.dirname = g_build_filename (GNOME_VFS_DATADIR,
+						       "application-registry",
+						       NULL);
 	gnome_registry_dir.system_dir = TRUE;
 	
-	user_registry_dir.dirname = g_strconcat (g_get_home_dir(), "/.gnome/application-info", NULL);
+	user_registry_dir.dirname = g_build_filename (g_get_home_dir(),
+						      ".gnome",
+						      "application-info",
+						      NULL);
 	user_registry_dir.system_dir = FALSE;
 
 	/* Make sure user directory exists */
-	if (mkdir (user_registry_dir.dirname, 0700) &&
+	if (g_mkdir (user_registry_dir.dirname, 0700) &&
 	    errno != EEXIST) {
 		g_warning("Could not create per-user Gnome application-registry directory: %s",
 			  user_registry_dir.dirname);
@@ -1072,9 +1078,6 @@ gnome_vfs_application_registry_shutdown (void)
 	g_free(user_registry_dir.dirname);
 	user_registry_dir.dirname = NULL;
 
-	g_list_free(current_lang);
-	current_lang = NULL;
-
 	gnome_vfs_application_registry_initialized = FALSE;
 }
 
@@ -1824,8 +1827,8 @@ gnome_vfs_application_registry_sync (void)
 
 	maybe_reload ();
 
-	file = g_strconcat (user_registry_dir.dirname, "/user.applications", NULL);
-	fp = fopen (file, "w");
+	file = g_build_filename (user_registry_dir.dirname, "user.applications", NULL);
+	fp = g_fopen (file, "w");
 
 	if ( ! fp) {
 		g_warning ("Cannot open '%s' for writing", file);
