@@ -36,18 +36,13 @@
 #include "gnome-vfs-ops.h"
 #include "gnome-vfs-result.h"
 #include "gnome-vfs-uri.h"
-#include <dirent.h>
-#include <regex.h>
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
 
-#define DEFAULT_DATE_TRACKER_INTERVAL	5	/* in milliseconds */
+#include <glib/gstdio.h>
 
-typedef struct {
-	char *mime_type;
-	regex_t regex;
-} RegexMimePair;
+#define DEFAULT_DATE_TRACKER_INTERVAL	5	/* in milliseconds */
 
 typedef struct {
 	char *file_path;
@@ -106,7 +101,15 @@ gnome_vfs_mime_type_from_name_or_default (const char *filename, const char *defa
 		return defaultv;
 	}
 
-	separator = g_utf8_strrchr (filename, -1, '/');
+	separator = strrchr (filename, '/');
+#ifdef G_OS_WIN32
+	{
+		const char *sep2 = strrchr (filename, '\\');
+		if (separator == NULL ||
+		    (sep2 != NULL && sep2 > separator))
+			separator = sep2;
+	}
+#endif
 	if (separator != NULL) {
 		separator++;
 		if (*separator == '\000')
@@ -190,14 +193,22 @@ const char *
 _gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer, const char *file_name, gboolean use_suffix)
 {
 	const char *result;
-	const char *zip_result;
+	const char *fn_result;
 
-	result = NULL;
+	result = fn_result = NULL;
+
+	/* Determine the mime type by filename as well, because
+	 * we need this for the subclassing check later
+	 */
+	if (file_name != NULL) {
+		fn_result = gnome_vfs_mime_type_from_name_or_default (file_name, NULL);
+	}
 
 	if (buffer != NULL) {
 		result = _gnome_vfs_read_mime_from_buffer (buffer);
-
+		
 		if (result != NULL && result != XDG_MIME_TYPE_UNKNOWN) {
+				
 			if ((strcmp (result, "application/x-ole-storage") == 0) ||
 			    (strcmp (result, "text/xml") == 0) ||
 			    (strcmp (result, "application/x-bzip") == 0) ||
@@ -207,19 +218,17 @@ _gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer, const char *
 				 * that extensions are more reliable than magic
 				 * typing. If the file has a suffix, then use 
 				 * the type from the suffix.
-		 		 *
-				 * FIXME bugzilla.gnome.org 46867:
-				 * Allow specific mime types to override 
-				 * magic detection
-				 */
-				
-				if (file_name != NULL) {
-					zip_result = gnome_vfs_mime_type_from_name_or_default (file_name, NULL);
-					if (zip_result != NULL && zip_result != XDG_MIME_TYPE_UNKNOWN) {
-						return zip_result;
-					}
+		 		 */
+
+				if (fn_result != NULL && fn_result != XDG_MIME_TYPE_UNKNOWN) {
+					result = fn_result;
 				}
+				
+			} else if (fn_result && fn_result != XDG_MIME_TYPE_UNKNOWN &&
+				   xdg_mime_mime_type_subclass (fn_result, result)) {
+				result = fn_result;
 			}
+			
 			return result;
 		}
 		
@@ -228,12 +237,8 @@ _gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer, const char *
 				/* Text file -- treat extensions as a more 
 				 * accurate source of type information.
 				 */
-				if (file_name != NULL) {
-					result = gnome_vfs_mime_type_from_name_or_default (file_name, NULL);
-				}
-	
-				if ((result != NULL) && (result != XDG_MIME_TYPE_UNKNOWN)) {
-					return result;
+				if ((fn_result != NULL) && (fn_result != XDG_MIME_TYPE_UNKNOWN)) {
+					return fn_result;
 				}
 
 				/* Didn't find an extension match, assume plain text. */
@@ -246,10 +251,9 @@ _gnome_vfs_get_mime_type_internal (GnomeVFSMimeSniffBuffer *buffer, const char *
 	}
 	
 	if (use_suffix &&
-	    (result == NULL || result == XDG_MIME_TYPE_UNKNOWN) &&
-	    file_name != NULL) {
+	    (result == NULL || result == XDG_MIME_TYPE_UNKNOWN)) {
 		/* No type recognized -- fall back on extensions. */
-		result = gnome_vfs_mime_type_from_name_or_default (file_name, NULL);
+		result = fn_result;
 	}
 	
 	if (result == NULL) {
@@ -349,7 +353,7 @@ gnome_vfs_get_file_mime_type_internal (const char *path, const struct stat *opti
 	result = NULL;
 
 	/* get the stat info if needed */
-	if (optional_stat_info == NULL && stat (path, &tmp_stat_buffer) == 0) {
+	if (optional_stat_info == NULL && g_stat (path, &tmp_stat_buffer) == 0) {
 		optional_stat_info = &tmp_stat_buffer;
 	}
 
@@ -363,8 +367,10 @@ gnome_vfs_get_file_mime_type_internal (const char *path, const struct stat *opti
 			return "x-special/device-block";
 		} else if (S_ISFIFO(optional_stat_info->st_mode)) {
 			return "x-special/fifo";
+#ifdef S_ISSOCK
 		} else if (S_ISSOCK(optional_stat_info->st_mode)) {
 			return "x-special/socket";
+#endif
 		} else {
 			/* unknown entry type, return generic file type */
 			return GNOME_VFS_MIME_TYPE_UNKNOWN;
@@ -380,7 +386,7 @@ gnome_vfs_get_file_mime_type_internal (const char *path, const struct stat *opti
 	}
 	
 	if (!suffix_only) {
-		file = fopen(path, "r");
+		file = g_fopen(path, "r");
 	}
 
 	if (file != NULL) {
@@ -639,7 +645,7 @@ static void
 file_date_record_update_mtime (FileDateRecord *record)
 {
 	struct stat s;
-	record->mtime = (stat (record->file_path, &s) != -1) ? s.st_mtime : 0;
+	record->mtime = (g_stat (record->file_path, &s) != -1) ? s.st_mtime : 0;
 }
 
 static FileDateRecord *
@@ -724,7 +730,7 @@ check_and_update_one (gpointer key, gpointer value, gpointer user_data)
 	record = (FileDateRecord *)value;
 	return_has_changed = (gboolean *)user_data;
 
-	if (stat (record->file_path, &s) != -1) {
+	if (g_stat (record->file_path, &s) != -1) {
 		if (s.st_mtime != record->mtime) {
 			record->mtime = s.st_mtime;
 			*return_has_changed = TRUE;
