@@ -25,10 +25,14 @@
 
 #include <string.h>
 #include <stdlib.h>
+#include <glib.h>
+#include <glib/gi18n-lib.h>
+#ifndef G_OS_WIN32
 #include <pthread.h>
-#include <libbonobo.h>
+#endif
+#include <bonobo/bonobo-exception.h>
+#include <bonobo/bonobo-object.h>
 #include <gconf/gconf-client.h>
-#include "gnome-vfs-i18n.h"
 #include "gnome-vfs-volume-monitor-private.h"
 #include "gnome-vfs-volume.h"
 #include "gnome-vfs-utils.h"
@@ -36,6 +40,8 @@
 #include "gnome-vfs-client.h"
 #include "gnome-vfs-private.h"
 
+#ifndef G_OS_WIN32
+
 #ifdef USE_VOLRMMOUNT
 
 static const char *volrmmount_locations [] = {
@@ -57,6 +63,7 @@ static const char *mount_known_locations [] = {
 };
 
 static const char *umount_known_locations [] = {
+	"/usr/bin/pumount", "/bin/pumount",
 	"/sbin/umount", "/bin/umount",
 	"/usr/sbin/umount", "/usr/bin/umount",
 	NULL
@@ -76,7 +83,7 @@ find_command (const char **known_locations)
 	int i;
 
 	for (i = 0; known_locations [i]; i++){
-		if (g_file_test (known_locations [i], G_FILE_TEST_EXISTS))
+		if (g_file_test (known_locations [i], G_FILE_TEST_IS_EXECUTABLE))
 			return known_locations [i];
 	}
 	return NULL;
@@ -261,7 +268,7 @@ mount_unmount_thread (void *arg)
 	}
 
 	if (info->should_eject) {
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__DragonFly__)
 	    	char *argv[5] = {
 		    	"cdcontrol",
 			"-f",
@@ -324,6 +331,7 @@ mount_unmount_thread (void *arg)
 	return NULL;
 }
 
+#endif	/* !G_OS_WIN32 */
 
 static void
 mount_unmount_operation (const char *mount_point,
@@ -335,6 +343,7 @@ mount_unmount_operation (const char *mount_point,
 			 GnomeVFSVolumeOpCallback  callback,
 			 gpointer                  user_data)
 {
+#ifndef G_OS_WIN32
 	const char *command = NULL;
 	const char *argument = NULL;
 	MountThreadInfo *mount_info;
@@ -390,6 +399,9 @@ mount_unmount_operation (const char *mount_point,
 	mount_info->user_data = user_data;
 	
 	pthread_create (&mount_thread, NULL, mount_unmount_thread, mount_info);
+#else
+	g_warning ("Not implemented: mount_unmount_operation()\n");
+#endif
 }
 
 
