@@ -293,13 +293,13 @@ typedef struct {
 	   
 } MethodSchemes;
 
-MethodSchemes supported_schemes[] = {
+static MethodSchemes supported_schemes[] = {
 
 	{"http",  FALSE, "http",  FALSE},
 	{"dav",   FALSE, "http",  TRUE},
 	{"https", TRUE,  "https", FALSE},
 	{"davs",  TRUE,  "https", TRUE},
-	{NULL,    FALSE,          FALSE}	   
+	{NULL,    FALSE,          NULL}	   
 };
 
 static char *
@@ -1316,7 +1316,7 @@ propfind_result (void *userdata, const char *href, const ne_prop_result_set *set
 
 G_LOCK_DEFINE (nst_lock);
 static GHashTable *neon_session_table;
-guint nst_tid;
+static guint nst_tid;
 
 typedef struct {
 	
@@ -1474,7 +1474,7 @@ neon_return_headers (ne_request *req, void *userdata, const ne_status *status)
 {	
 	GnomeVFSModuleCallbackReceivedHeadersIn in_args;
 	GnomeVFSModuleCallbackReceivedHeadersOut out_args;
-	GList **headers, *iter;
+	GList **headers;
 	GnomeVFSURI *uri;
 	ne_session *session;
 
@@ -1498,14 +1498,6 @@ neon_return_headers (ne_request *req, void *userdata, const ne_status *status)
 					&in_args, sizeof (in_args),
 					&out_args, sizeof (out_args));
 	
-	for (iter = *headers; iter; iter = iter->next) {
-		DEBUG_HTTP_3 ("Headers returned %s,", (char *) iter->data);
-		g_free (iter->data);
-	}
-
-	g_list_free (*headers);
-	g_free (headers);
-
 	ne_set_request_private (req, "Headers Returned", "TRUE");
 	
 	DEBUG_HTTP_FUNC (0);
@@ -1568,6 +1560,30 @@ neon_setup_headers (ne_request *req, void *userdata, ne_buffer *header)
 	DEBUG_HTTP_FUNC (0);
 }
 
+static void 
+neon_free_headers (ne_request *req, void *userdata)
+{
+	GList **headers, *iter;
+	
+	DEBUG_HTTP_FUNC (1);
+	headers = ne_get_request_private (req, "Headers");
+	
+	if (headers == NULL) {
+		return;	
+	}
+	
+	for (iter = *headers; iter; iter = iter->next) {
+		DEBUG_HTTP_3 ("Freeing Header  %s,", (char *) iter->data);
+		g_free (iter->data);
+	}
+
+	g_list_free (*headers);
+	g_free (headers);
+	
+	ne_set_request_private (req, "Headers", NULL);
+	DEBUG_HTTP_FUNC (0);
+}
+
 /* ************************************************************************** */
 /* Http context */
 typedef struct {
@@ -1648,6 +1664,7 @@ http_aquire_connection (HttpContext *context)
 	ne_set_session_private (session, "GnomeVFSURI", context->uri);
 	ne_hook_pre_send (session, neon_setup_headers, NULL);
 	ne_hook_post_send (session, neon_return_headers, NULL);
+	ne_hook_destroy_request (session, neon_free_headers, NULL);
 
 	if (proxy_for_uri (top_uri, &proxy)) {
 		HttpAuthInfo *proxy_auth;
@@ -1786,6 +1803,8 @@ http_follow_redirect (HttpContext *context)
 	
 	NE_FREE (redir_texturi);
 	
+	/* see if redirect is to another host:port pair so we need a new
+	 * connection/session */
 	if (! http_session_uri_equal (context->uri, new_uri)) {
 		
 		/* release connection */
@@ -1793,7 +1812,6 @@ http_follow_redirect (HttpContext *context)
 		context->session = NULL;
 		
 		http_context_set_uri (context, new_uri);
-		
 		result = http_aquire_connection (context);
 		
 	} else {
@@ -1802,6 +1820,10 @@ http_follow_redirect (HttpContext *context)
 		ne_set_session_private (context->session, "GnomeVFSURI", context->uri);
 		result = GNOME_VFS_OK;
 	}
+	
+	/* uri got dupped by http_context_set_uri () */
+	gnome_vfs_uri_unref (new_uri);
+	
 	DEBUG_HTTP ("[Redirec] Redirect result: %s", 
 		    gnome_vfs_result_to_string (result));
 	return result;
@@ -1879,8 +1901,11 @@ http_get_file_info (HttpContext *context, GnomeVFSFileInfo *info)
 	if (res == NE_REDIRECT) {
 		result = http_follow_redirect (context);
 		
-		if (result == GNOME_VFS_OK)
+		if (result == GNOME_VFS_OK) {
+			ne_request_destroy (req);
+			req = NULL;
 			goto head_start;
+		}
 	}
 
 
@@ -1989,6 +2014,7 @@ http_options (HttpContext *hctx)
 	}
 
 	result = resolve_result (res, req);
+	ne_request_destroy (req);
 	
 	if (result == GNOME_VFS_OK) {
 		
@@ -2421,6 +2447,7 @@ do_create (GnomeVFSMethod	 *method,
 		
 		if (result != GNOME_VFS_ERROR_NOT_FOUND) {
 			http_file_handle_destroy (handle);
+			ne_request_destroy (req);
 			return GNOME_VFS_ERROR_FILE_EXISTS;
 		}
 	}
@@ -2429,6 +2456,7 @@ do_create (GnomeVFSMethod	 *method,
 	
 	res = ne_request_dispatch (req);
 	result 	= resolve_result (res, req);
+	ne_request_destroy (req);
 	
 	if (result == GNOME_VFS_OK && mode != GNOME_VFS_OPEN_NONE) {
 
@@ -2483,13 +2511,14 @@ do_close (GnomeVFSMethod 	*method,
 		   
 		DEBUG_HTTP ("[PUT] Filesize: %d", len);
 		
-		ne_set_request_body_buffer (req, data, len);
+		ne_set_request_body_buffer (req, (char *)data, len);
 		res = ne_request_dispatch (req);
 
 		result = resolve_result (res, req);
 		DEBUG_HTTP ("[PUT] returned: %d, %d (%s)", res, 
 				ne_get_status (req)->code, 
 				gnome_vfs_result_to_string (result));
+		ne_request_destroy (req);
 	}
 	
 	
@@ -2696,7 +2725,7 @@ do_seek (GnomeVFSMethod	      *method,
 static GnomeVFSResult
 do_tell (GnomeVFSMethod       *method,
 	 GnomeVFSMethodHandle *method_handle,
-	 GnomeVFSFileOffset   *offset_return)
+	 GnomeVFSFileSize   *offset_return)
 {
 	HttpFileHandle *handle;
 	
@@ -2942,6 +2971,8 @@ do_make_directory (GnomeVFSMethod  *method,
 		result = http_follow_redirect (hctx);
 		
 		if (result == GNOME_VFS_OK)
+			ne_request_destroy (req);
+			req = NULL;
 			goto mkcol_start;
 	} else if (res == NE_OK) {
 		const ne_status *status = ne_get_status (req);
@@ -3073,13 +3104,16 @@ do_move (GnomeVFSMethod  *method,
 	if (res == NE_REDIRECT) {
 		result = http_follow_redirect (hctx);
 
-		/* TODO: Do we leak req here? */
-		if (result == GNOME_VFS_OK)
+		if (result == GNOME_VFS_OK) {
+			ne_request_destroy (req);
+			req = NULL;
 			goto head_start;
+		}
 	} else {
 		result = resolve_result (res, req);
 	}
 
+	ne_request_destroy (req);
 	http_context_free (hctx);
 	
 	DEBUG_HTTP_FUNC (0);
@@ -3157,10 +3191,6 @@ do_set_file_info (GnomeVFSMethod 		*method,
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 	}
 	
-	/* this wouldn't rename but move the file */
-	if (g_strrstr (info->name, "/"))
-		return GNOME_VFS_ERROR_BAD_PARAMETERS;
-
 	parent_uri = gnome_vfs_uri_get_parent (uri);
 	
 	if (parent_uri == NULL) {
