@@ -31,7 +31,6 @@
 #include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-directory.h>
-#include <libgnomevfs/gnome-vfs-i18n.h>
 #include <libgnomevfs/gnome-vfs-module-shared.h>
 #include <libgnomevfs/gnome-vfs-module.h>
 #include <libgnomevfs/gnome-vfs-volume-monitor.h>
@@ -39,6 +38,8 @@
 #include <libgnomevfs/gnome-vfs-dns-sd.h>
 #include <libgnomevfs/gnome-vfs-monitor-private.h>
 
+#include <glib/gi18n-lib.h>
+
 #define PATH_GCONF_GNOME_VFS_SMB "/system/smb"
 #define PATH_GCONF_GNOME_VFS_SMB_WORKGROUP "/system/smb/workgroup"
 #define PATH_GCONF_GNOME_VFS_DNS_SD "/system/dns_sd"
@@ -672,7 +673,7 @@ do_seek (GnomeVFSMethod *method,
 static GnomeVFSResult
 do_tell (GnomeVFSMethod *method,
 	 GnomeVFSMethodHandle *method_handle,
-	 GnomeVFSFileOffset *offset_return)
+	 GnomeVFSFileSize *offset_return)
 {
 	FileHandle *file_handle;
 
