@@ -51,7 +51,6 @@
 
 #include <config.h>
 
-#include <libgnomevfs/gnome-vfs-i18n.h>
 #include <libgnomevfs/gnome-vfs-context.h>
 #include <libgnomevfs/gnome-vfs-method.h>
 #include <libgnomevfs/gnome-vfs-module.h>
@@ -67,6 +66,8 @@
 #include <errno.h>
 #include <stdlib.h>
 
+#include <glib/gi18n-lib.h>
+
 #include "sftp.h"
 #include "pty-open.h"
 
@@ -161,14 +162,11 @@ static gboolean sftp_connection_process_errors (GIOChannel *channel,
 						GIOCondition cond,
 						GnomeVFSResult *status);
 
-
-static guint inited_buffers = 0;
-
 typedef struct
 {
-	guchar  *base;
-	guchar  *read_ptr;
-	guchar  *write_ptr;
+	char  *base;
+	char  *read_ptr;
+	char  *write_ptr;
 	gint     alloc;
 } Buffer;
 
@@ -211,8 +209,6 @@ buffer_init (Buffer *buf)
 	buf->read_ptr = buf->base + sizeof (guint);
 	buf->write_ptr = buf->base + sizeof (guint);
 	buf->alloc = INIT_BUFFER_ALLOC;
-
-	++inited_buffers;
 }
 
 static void
@@ -220,16 +216,14 @@ buffer_free (Buffer *buf)
 {
 	g_return_if_fail (buf != NULL);
 
-	if (inited_buffers == 0) {
+	if (buf->base == NULL) {
 		g_critical ("No initialized buffers present. Something is being double-freed");
 		return;
 	}
 
 	g_free (buf->base);
-	buf->base = buf->read_ptr = buf->write_ptr = (gpointer) 0xdeadbeef;
+	buf->base = buf->read_ptr = buf->write_ptr = NULL;
 	buf->alloc = 0;
-
-	--inited_buffers;
 }
 
 static void
@@ -239,7 +233,6 @@ buffer_check_alloc (Buffer *buf, guint32 size)
 
 	g_return_if_fail (buf != NULL);
 	g_return_if_fail (buf->base != NULL);
-	g_return_if_fail (buf->base != (gpointer) 0xdeadbeef);
 
 	DEBUG4 (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Allocating %p to %d", buf, size));
 
@@ -258,7 +251,6 @@ buffer_clear (Buffer *buf)
 {
 	g_return_if_fail (buf != NULL);
 	g_return_if_fail (buf->base != NULL);
-	g_return_if_fail (buf->base != (gpointer) 0xdeadbeef);
 
 	DEBUG4 (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Clearing %p", buf));
 
@@ -272,7 +264,6 @@ buffer_read (Buffer *buf, gpointer data, guint32 size)
 
 	g_return_if_fail (buf != NULL);
 	g_return_if_fail (buf->base != NULL);
-	g_return_if_fail (buf->base != (gpointer) 0xdeadbeef);
 
 	DEBUG4 (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Reading %d from %p to %p", size, buf, data));
 
@@ -289,7 +280,6 @@ buffer_write (Buffer *buf, gconstpointer data, guint32 size)
 {
 	g_return_if_fail (buf != NULL);
 	g_return_if_fail (buf->base != NULL);
-	g_return_if_fail (buf->base != (gpointer) 0xdeadbeef);
 
 	DEBUG4 (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Writing %d from %p to %p", size, data, buf));
 
@@ -308,7 +298,6 @@ buffer_send (Buffer *buf, int fd)
 
 	g_return_val_if_fail (buf != NULL, GNOME_VFS_ERROR_INTERNAL);
 	g_return_val_if_fail (buf->base != NULL, GNOME_VFS_ERROR_INTERNAL);
-	g_return_val_if_fail (buf->base != (gpointer) 0xdeadbeef, GNOME_VFS_ERROR_INTERNAL);
 
 	DEBUG2 (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s: Sending message of length %d from %p to %d",
 		       __FUNCTION__, len, buf, fd));
@@ -344,7 +333,6 @@ buffer_recv (Buffer *buf, int fd)
 
 	g_return_val_if_fail (buf != NULL, GNOME_VFS_ERROR_INTERNAL);
 	g_return_val_if_fail (buf->base != NULL, GNOME_VFS_ERROR_INTERNAL);
-	g_return_val_if_fail (buf->base != (gpointer) 0xdeadbeef, GNOME_VFS_ERROR_INTERNAL);
 
 	DEBUG2 (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s: Receiving message from %d to %p",
 		       __FUNCTION__, fd, buf));
@@ -394,7 +382,6 @@ buffer_read_gchar (Buffer *buf)
 
 	g_return_val_if_fail (buf != NULL, 0);
 	g_return_val_if_fail (buf->base != NULL, 0);
-	g_return_val_if_fail (buf->base != (gpointer) 0xdeadbeef, 0);
 
 	buffer_read (buf, &data, sizeof (gchar));
 
@@ -410,7 +397,6 @@ buffer_read_gint32 (Buffer *buf)
 
 	g_return_val_if_fail (buf != NULL, 0);
 	g_return_val_if_fail (buf->base != NULL, 0);
-	g_return_val_if_fail (buf->base != (gpointer) 0xdeadbeef, 0);
 
 	buffer_read (buf, &data, sizeof (gint32));
 
@@ -427,7 +413,6 @@ buffer_read_gint64 (Buffer *buf)
 
 	g_return_val_if_fail (buf != NULL, 0);
 	g_return_val_if_fail (buf->base != NULL, 0);
-	g_return_val_if_fail (buf->base != (gpointer) 0xdeadbeef, 0);
 
 	buffer_read (buf, &data, sizeof (gint64));
 
@@ -445,7 +430,6 @@ buffer_read_block (Buffer *buf, gint32 *p_len)
 
 	g_return_val_if_fail (buf != NULL, NULL);
 	g_return_val_if_fail (buf->base != NULL, NULL);
-	g_return_val_if_fail (buf->base != (gpointer) 0xdeadbeef, NULL);
 
 	if (p_len == NULL)
 		p_len = &len;
@@ -467,7 +451,6 @@ buffer_read_string (Buffer *buf, gint32 *p_len)
 
 	g_return_val_if_fail (buf != NULL, NULL);
 	g_return_val_if_fail (buf->base != NULL, NULL);
-	g_return_val_if_fail (buf->base != (gpointer) 0xdeadbeef, NULL);
 
 	if (p_len == NULL)
 		p_len = &len;
@@ -493,7 +476,6 @@ buffer_read_file_info (Buffer *buf, GnomeVFSFileInfo *info)
 
 	g_return_if_fail (buf != NULL);
 	g_return_if_fail (buf->base != NULL);
-	g_return_if_fail (buf->base != (gpointer) 0xdeadbeef);
 
 	flags = buffer_read_gint32 (buf);
 
@@ -551,7 +533,6 @@ buffer_write_gchar (Buffer *buf, gchar data)
 {
 	g_return_if_fail (buf != NULL);
 	g_return_if_fail (buf->base != NULL);
-	g_return_if_fail (buf->base != (gpointer) 0xdeadbeef);
 
 	DEBUG4 (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Writing byte %d to %p", data, buf));
 
@@ -565,7 +546,6 @@ buffer_write_gint32 (Buffer *buf, gint32 data)
 
 	g_return_if_fail (buf != NULL);
 	g_return_if_fail (buf->base != NULL);
-	g_return_if_fail (buf->base != (gpointer) 0xdeadbeef);
 
 	DEBUG4 (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Writing word %d to %p", data, buf));
 
@@ -580,7 +560,6 @@ buffer_write_gint64 (Buffer *buf, gint64 data)
 
 	g_return_if_fail (buf != NULL);
 	g_return_if_fail (buf->base != NULL);
-	g_return_if_fail (buf->base != (gpointer) 0xdeadbeef);
 
 	DEBUG4 (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Writing doubleword %lld to %p", data, buf));
 
@@ -593,7 +572,6 @@ buffer_write_block (Buffer *buf, gconstpointer ptr, gint32 len)
 {
 	g_return_if_fail (buf != NULL);
 	g_return_if_fail (buf->base != NULL);
-	g_return_if_fail (buf->base != (gpointer) 0xdeadbeef);
 
 	DEBUG4 (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Writing block of length %d to %p", len, buf));
 
@@ -608,7 +586,6 @@ buffer_write_string (Buffer *buf, const gchar *data)
 
 	g_return_if_fail (buf != NULL);
 	g_return_if_fail (buf->base != NULL);
-	g_return_if_fail (buf->base != (gpointer) 0xdeadbeef);
 
 	len = strlen (data);
 
@@ -627,7 +604,6 @@ buffer_write_file_info (Buffer *buf, const GnomeVFSFileInfo *info, GnomeVFSSetFi
 
 	g_return_if_fail (buf != NULL);
 	g_return_if_fail (buf->base != NULL);
-	g_return_if_fail (buf->base != (gpointer) 0xdeadbeef);
 
 	DEBUG4 (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Writing file info to %p", buf));
 
@@ -726,7 +702,7 @@ iobuf_read_handle (int fd, gchar **handle, guint expected_id, guint32 *len)
 		g_critical ("Expected SSH2_FXP_HANDLE(%u) packet, got %u",
 			    SSH2_FXP_HANDLE, type);
 
-	*handle = buffer_read_block (&msg, len);
+	*handle = buffer_read_block (&msg, (gint32 *)len);
 
 	buffer_free (&msg);
 
@@ -1086,7 +1062,7 @@ sftp_connect (SftpConnection **connection, const GnomeVFSURI *uri)
 	guint           last_arg, i;
 	gboolean        full_auth;
 	gboolean        done_auth;
-	gboolean	save_password;
+	gboolean	save_password = FALSE;
 	Buffer          msg;
 	gchar           type;
 	char *password = NULL;
@@ -1245,14 +1221,14 @@ sftp_connect (SftpConnection **connection, const GnomeVFSURI *uri)
 		FD_SET (in_fd, &ifds);
 		FD_SET (prompt_fd, &ifds);
 
-		tv.tv_sec = 10;
+		tv.tv_sec = 20;
 		tv.tv_usec = 0;
 
 		ret = select (MAX (in_fd, prompt_fd)+1, &ifds, NULL, NULL, &tv);
 
 		if (ret <= 0) {
 			/* Timeout */
-			res = GNOME_VFS_ERROR_IO;
+			res = GNOME_VFS_ERROR_TIMEOUT;
 			goto bail;
 		}
 
@@ -1714,6 +1690,7 @@ get_real_path (SftpConnection *conn, const gchar *path, gchar **realpath)
 
 	if (res != GNOME_VFS_OK) {
 		g_critical ("Error receiving message: %d", res);
+		buffer_free (&msg);
 		return res;
 	}
 
@@ -1809,8 +1786,9 @@ do_open (GnomeVFSMethod        *method,
 	buffer_write_file_info (&msg, &info, GNOME_VFS_SET_FILE_INFO_NONE);
 
 	buffer_send (&msg, conn->out_fd);
-
-	sftp_res = iobuf_read_handle (conn->in_fd, &sftp_handle, id, &sftp_handle_len);
+	buffer_free (&msg);
+	
+	sftp_res = iobuf_read_handle (conn->in_fd, &sftp_handle, id, (guint32 *)&sftp_handle_len);
 
 	if (sftp_res == SSH2_FX_OK) {
 		handle = g_new0 (SftpOpenHandle, 1);
@@ -2047,6 +2025,7 @@ do_read (GnomeVFSMethod       *method,
 		outstanding--;
 
 		if (result != GNOME_VFS_OK) {
+			buffer_free (&msg);
 			sftp_connection_unlock (handle->connection);
 			return result;
 		}
@@ -2303,7 +2282,7 @@ do_seek (GnomeVFSMethod       *method,
 static GnomeVFSResult
 do_tell (GnomeVFSMethod       *method,
 	 GnomeVFSMethodHandle *method_handle,
-	 GnomeVFSFileOffset   *offset_return)
+	 GnomeVFSFileSize   *offset_return)
 {
 	SftpOpenHandle *handle;
 
@@ -2565,12 +2544,12 @@ do_read_directory (GnomeVFSMethod       *method,
 
 	if (type == SSH2_FXP_STATUS) {
 		status = buffer_read_gint32 (&msg);
-
+		buffer_free (&msg);
+		
 		if (status == SSH2_FX_EOF || SSH2_FX_OK) {
 			DEBUG (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
 				      "%s: End of directory reached (EOF status)",
 				      __FUNCTION__));
-			buffer_free (&msg);
 			sftp_connection_unlock (handle->connection);
 			return GNOME_VFS_ERROR_EOF;
 		} else {
@@ -2652,6 +2631,8 @@ do_read_directory (GnomeVFSMethod       *method,
 		return GNOME_VFS_ERROR_PROTOCOL_ERROR;
 	}
 
+	buffer_free (&msg);
+	
 	if (handle->info_read_ptr < handle->info_write_ptr) {
 		gnome_vfs_file_info_copy (file_info, &(handle->info[handle->info_read_ptr]));
 		g_free (handle->info[handle->info_read_ptr].name);
@@ -2754,8 +2735,6 @@ do_move (GnomeVFSMethod  *method,
 	res = sftp_get_connection (&conn, old_uri);
 	if (res != GNOME_VFS_OK) return res;
 
-	buffer_init (&msg);
-
 	old_path = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (old_uri), NULL);
 	if (old_path == NULL)
 		return GNOME_VFS_ERROR_INVALID_URI;
@@ -2778,6 +2757,7 @@ do_move (GnomeVFSMethod  *method,
 			goto bail;
 	}
 
+	buffer_init (&msg);
 	buffer_write_gchar (&msg, SSH2_FXP_RENAME);
 	buffer_write_gint32 (&msg, id);
 	buffer_write_string (&msg, old_path);
@@ -2815,8 +2795,6 @@ do_rename (GnomeVFSMethod  *method,
 	res = sftp_get_connection (&conn, old_uri);
 	if (res != GNOME_VFS_OK) return res;
 
-	buffer_init (&msg);
-
 	old_path = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (old_uri), NULL);
 	if (old_path == NULL)
 		return GNOME_VFS_ERROR_INVALID_URI;
@@ -2831,6 +2809,7 @@ do_rename (GnomeVFSMethod  *method,
 
 	id = sftp_connection_get_id (conn);
 
+	buffer_init (&msg);
 	buffer_write_gchar (&msg, SSH2_FXP_RENAME);
 	buffer_write_gint32 (&msg, id);
 	buffer_write_string (&msg, old_path);
@@ -2980,14 +2959,13 @@ do_create_symlink (GnomeVFSMethod   *method,
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 	}
 	
-	buffer_init (&msg);
-
 	path = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (uri), NULL);
 	if (path == NULL)
 		return GNOME_VFS_ERROR_INVALID_URI;
 
 	id = sftp_connection_get_id (conn);
-
+	
+	buffer_init (&msg);
 	buffer_write_gchar (&msg, SSH2_FXP_SYMLINK);
 	buffer_write_gint32 (&msg, id);
 	buffer_write_string (&msg, path);
@@ -3055,9 +3033,4 @@ vfs_module_shutdown (GnomeVFSMethod *method)
 	G_LOCK (sftp_connection_table);
 	g_hash_table_foreach_remove (sftp_connection_table, (GHRFunc) close_thunk, NULL);
 	G_UNLOCK (sftp_connection_table);
-
-	if (inited_buffers != 0)
-		g_critical ("%d buffers leaked", inited_buffers);
-	else
-		DEBUG (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "inited_buffers is ok"));
 }
