@@ -41,13 +41,14 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
-#include <libgnomevfs/gnome-vfs-i18n.h>
 #include <libgnomevfs/gnome-vfs-method.h>
 #include <libgnomevfs/gnome-vfs-module.h>
 #include <libgnomevfs/gnome-vfs-module-shared.h>
 #include <libgnomevfs/gnome-vfs-module-callback-module-api.h>
 #include <libgnomevfs/gnome-vfs-standard-callbacks.h>
 
+#include <glib/gi18n-lib.h>
+
 #include <libsmbclient.h>
 
 int smbc_remove_unused_server(SMBCCTX * context, SMBCSRV * srv);
@@ -133,6 +134,8 @@ typedef struct _SmbAuthContext {
 
 	gboolean cache_added;           /* Set when cache is added to during authentication */
 	gboolean cache_used;            /* Set when cache is used during authentication */
+
+	guint prompt_flags;           	/* Various URI flags set by initial_authentication */
 	
 } SmbAuthContext;
 
@@ -366,13 +369,14 @@ add_cached_server (SMBCCTX *context, SMBCSRV *new,
 }
 
 static SMBCSRV *
-get_cached_server (SMBCCTX * context,
-		   const char *server_name, const char *share_name,
-		   const char *domain, const char *username)
+find_cached_server (const char *server_name, const char *share_name,
+                    const char *domain, const char *username)
 {
 	SmbServerCacheEntry entry;
 	SmbServerCacheEntry *res;
 
+	DEBUG_SMB(("find_cached_server: server: %s, share: %s, domain: %s, user: %s\n", server_name, share_name, domain, username));
+
 	entry.server_name = (char *)server_name;
 	entry.share_name = (char *)share_name;
 	entry.domain = (char *)domain;
@@ -381,11 +385,26 @@ get_cached_server (SMBCCTX * context,
 	res = g_hash_table_lookup (server_cache, &entry);
 
 	if (res != NULL) {
-		DEBUG_SMB(("got cached server: server: %s, share: %s, domain: %s, user: %s\n",
-		   	  server_name, share_name, domain, username));
-		current_auth_context->cache_used = TRUE;
 		res->last_time = time (NULL);
 		return res->server;
+	} 
+
+	return NULL;
+}
+
+static SMBCSRV *
+get_cached_server (SMBCCTX * context,
+		   const char *server_name, const char *share_name,
+		   const char *domain, const char *username)
+{
+	SMBCSRV *srv;
+	
+	srv = find_cached_server (server_name, share_name, domain, username);
+	if (srv != NULL) {
+		DEBUG_SMB(("got cached server: server: %s, share: %s, domain: %s, user: %s\n",
+		   	  	  server_name, share_name, domain, username));
+		current_auth_context->cache_used = TRUE;
+		return srv;
 	}
 	return NULL;
 }
@@ -698,7 +717,7 @@ update_user_cache (SmbAuthContext *actx, gboolean with_share)
 }
 
 static gboolean
-lookup_user_cache (SmbAuthContext *actx, gboolean with_share, gboolean with_user)
+lookup_user_cache (SmbAuthContext *actx, gboolean with_share)
 {
         SmbCachedUser *user;
         gchar *key;
@@ -710,10 +729,13 @@ lookup_user_cache (SmbAuthContext *actx, gboolean with_share, gboolean with_user
         g_free (key);
        
         if (user) {
-                /* If the caller specified we already have a user name double check that... */
-                if (with_user && (!string_compare (user->username, actx->use_user) ||
-                                  !string_compare (user->domain, actx->use_domain)))
-                    return FALSE;
+                /* If we already have a user name or domain double check that... */
+		if (!(actx->prompt_flags & GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_NEED_USERNAME) &&
+		    !string_compare(user->username, actx->use_user))
+			return FALSE;
+		if (!(actx->prompt_flags & GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_NEED_DOMAIN) &&
+		    !string_compare(user->domain, actx->use_domain))
+			return FALSE;
 
                 actx->use_user = string_realloc (actx->use_user, user->username);
                 actx->use_domain = string_realloc (actx->use_domain, user->domain);
@@ -740,6 +762,8 @@ initial_authentication (SmbAuthContext *actx)
 	DEBUG_SMB(("[auth] Initial authentication lookups\n"));
 
 	toplevel_uri =	(GnomeVFSToplevelURI*)actx->uri;
+	actx->prompt_flags = GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_NEED_USERNAME |
+		    	     GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_NEED_DOMAIN;
 	
 	/* Try parsing a user and domain out of the URI */
 	if (toplevel_uri && toplevel_uri->user_name != NULL && 
@@ -752,19 +776,28 @@ initial_authentication (SmbAuthContext *actx)
 					    	  	      tmp - toplevel_uri->user_name);
 			g_free (actx->use_user);
 			actx->use_user = string_dup_nzero (tmp + 1);
+
+			if (actx->use_domain != NULL)
+				actx->prompt_flags &= ~GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_NEED_DOMAIN;
+			if (actx->use_user != NULL)
+				actx->prompt_flags &= ~GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_NEED_USERNAME;
+
 			DEBUG_SMB(("[auth] User from URI: %s@%s\n", actx->use_user, actx->use_domain));
 		} else {
 			g_free (actx->use_user);
 			actx->use_user = string_dup_nzero (toplevel_uri->user_name);
 			g_free (actx->use_domain);
 			actx->use_domain = NULL;
+
+			if (actx->use_user != NULL)
+				actx->prompt_flags &= ~GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_NEED_USERNAME;
 			DEBUG_SMB(("[auth] User from URI: %s\n", actx->use_user));
 		}
                 preset_user = TRUE;
 	} 
 
-        if (lookup_user_cache (actx, TRUE, preset_user) ||
-            lookup_user_cache (actx, FALSE, preset_user))
+        if (lookup_user_cache (actx, TRUE) ||
+            lookup_user_cache (actx, FALSE))
                 found_user = TRUE;
 
         if (found_user || preset_user) {
@@ -785,10 +818,8 @@ initial_authentication (SmbAuthContext *actx)
                         }
                 }
                 
-                /* Server is in cache already, no need to get password */
+                /* Server is in cache already */
                 if (server != NULL) {
-        		g_free (actx->use_password);
-        		actx->use_password = g_strdup ("");
         		DEBUG_SMB(("[auth] Using connection for '%s@%s' from our server cache\n", actx->use_user, actx->use_domain));
                         found_user = TRUE;
         	}
@@ -889,10 +920,7 @@ prompt_authentication (SmbAuthContext *actx)
 	in_args.flags = GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_NEED_PASSWORD | GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_SAVING_SUPPORTED;
 	if (actx->state & SMB_AUTH_STATE_PROMPTED)
 		in_args.flags |= GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_PREVIOUS_ATTEMPT_FAILED;
-	if (!actx->uri || ((GnomeVFSToplevelURI*)actx->uri)->user_name == NULL) {
-		in_args.flags |= GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_NEED_USERNAME |
-	  			 GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION_NEED_DOMAIN;
-	}
+	in_args.flags |= actx->prompt_flags;
 
 	in_args.uri = get_auth_display_uri (actx, FALSE);
 	in_args.protocol = "smb";
@@ -1193,6 +1221,7 @@ auth_callback (const char *server_name, const char *share_name,
 {
 	/* IMPORTANT: We are IN the global lock */
 	SmbAuthContext *actx;
+	SMBCSRV *server;
 	
 	DEBUG_SMB (("[auth] auth_callback called: server: %s share: %s\n",
 		    server_name, share_name));
@@ -1240,6 +1269,19 @@ auth_callback (const char *server_name, const char *share_name,
 	/* Put in the default workgroup if none specified */
 	if (domain_out[0] == 0 && smb_context->workgroup)
 		strncpy (domain_out, smb_context->workgroup, domainmaxlen);
+
+	/* 
+	 * If authentication is requested a second time on a server we've 
+	 * already cached, then obviously it was invalid. Remove it. Yes, 
+	 * this doesn't make much sense, but such is life with libsmbclient.
+	 */
+	if ((actx->state & SMB_AUTH_STATE_PROMPTED) && actx->res != GNOME_VFS_OK) {
+		server = find_cached_server (server_name, share_name, domain_out, username_out);
+		if (server) {
+			DEBUG_SMB (("[auth] auth_callback. Remove the wrong server entry from server_cache.\n"));
+			g_hash_table_foreach_remove (server_cache, remove_server, server);
+		}
+	}
 }
 
 static char *
@@ -1446,7 +1488,7 @@ do_read (GnomeVFSMethod *method,
 	 GnomeVFSContext *context)
 {
 	FileHandle *handle = (FileHandle *)method_handle;
-	GnomeVFSResult res = GNOME_VFS_OK;;
+	GnomeVFSResult res = GNOME_VFS_OK;
 	SmbAuthContext actx;
 	ssize_t n = 0;
 
@@ -2119,7 +2161,7 @@ do_seek (GnomeVFSMethod *method,
 static GnomeVFSResult
 do_tell (GnomeVFSMethod *method,
 		GnomeVFSMethodHandle *method_handle,
-		GnomeVFSFileOffset *offset_return)
+		GnomeVFSFileSize *offset_return)
 {
 	FileHandle *handle = (FileHandle *)method_handle;
 	SmbAuthContext actx;
